@@ -4,8 +4,11 @@ from python_grains.profiles.exceptions import InvalidQuery, InvalidAudienceConfi
 DEFAULT_REDIS_CLIENT = None
 DEFAULT_DJANGO_AUDIENCE_MODEL = None
 DEFAULT_DJANGO_CONNECTION_TUPLE = (None, None)
+DEFAULT_DJANGO_MAX_FUNC = None
 DEFAULT_AUDIENCE_TTL = 14 * 24 * 60 * 60
+DEFAULT_AUDIENCE_DOMAINS = []
 DEFAULT_DOMAIN = None
+DEFAULT_AUDIENCE_ID_PREFIX = 'hic_au_'
 
 ALLOWED_FIELD_TYPES = {
     'int': {
@@ -88,19 +91,21 @@ class AudienceQuery(object):
                  query,
                  op_c1=None,
                  name=None,
-                 au_id=None):
+                 au_id=None,
+                 raw_query=None):
 
         self.op_c1 = op_c1
         self.query = query
         self.name = name
         self.au_id = au_id
+        self.raw_query = raw_query
 
     @classmethod
     def from_raw_data(cls, raw_data, op_c1=None, name=None):
         query_data = raw_data['query']
         au_id = raw_data['au_id']
         query = cls.parse(query_data)
-        return cls(query=query, au_id=au_id, op_c1=op_c1, name=name)
+        return cls(query=query, au_id=au_id, op_c1=op_c1, name=name, raw_query=query_data)
 
     @classmethod
     def parse_c1_operator_argument(cls,
@@ -204,9 +209,12 @@ class Audiences(object):
 
     django_audience_model = DEFAULT_DJANGO_AUDIENCE_MODEL
     django_connection_tuple = DEFAULT_DJANGO_CONNECTION_TUPLE
+    django_max_func = DEFAULT_DJANGO_MAX_FUNC
     audience_ttl = DEFAULT_AUDIENCE_TTL
     redis_client = DEFAULT_REDIS_CLIENT
     domain = DEFAULT_DOMAIN
+    audience_domains = DEFAULT_AUDIENCE_DOMAINS
+    default_audience_id_prefix = DEFAULT_AUDIENCE_ID_PREFIX
 
     def __init__(self,
                  au_domain=None):
@@ -214,13 +222,62 @@ class Audiences(object):
         self.au_domain = au_domain or '_global'
         self.queries = self.get_queries()
 
-    def run(self, data):
+    def add(self,
+            query,
+            name='',
+            description='',
+            active=True,
+            id_prefix=None,
+            id_num=None):
+
+        AudienceQuery.parse(query)
+
+        id_prefix = id_prefix or self.default_audience_id_prefix
+
+        if id_num is None:
+            q = self.django_audience_model.objects.filter(id_prefix=id_prefix).aggregate(self.django_max_func('id_num'))
+            id_num_max = q['id_num__max']
+
+            if id_num_max is None:
+                id_num_max = 0
+            id_num = id_num_max + 1
+
+        audience = self.django_audience_model(
+            id_num=id_num,
+            id_prefix=id_prefix,
+            query=json.dumps(query),
+            name=name,
+            description=description,
+            active=active,
+            domain=self.au_domain)
+
+        audience.save()
+        self.build_cache()
+        return audience
+
+    def run(self,
+            data):
         return [query.au_id for query in self.queries if query.run(data)]
 
     @classmethod
-    def list(cls):
+    def parse_query(cls, query):
+        AudienceQuery.parse(raw_data=query)
+
+    @classmethod
+    def n_active(cls, au_domain):
+        return cls.django_audience_model.objects.filter(domain=au_domain, active=True).count()
+
+    @classmethod
+    def list(cls, au_domain=None):
+
+        if au_domain is None:
+            audiences = [audience.repr_data for audience in cls.django_audience_model.objects.all()]
+        else:
+            audiences = [audience.repr_data for audience in cls.django_audience_model.objects.filter(domain=au_domain)]
+            if not au_domain == '_global':
+                audiences.extend(
+                    [audience.repr_data for audience in cls.django_audience_model.objects.filter(domain='_global')])
 
-        audiences = [audience.repr_data for audience in cls.django_audience_model.objects.all()]
         return audiences
 
     def get_queries(self,
@@ -233,7 +290,7 @@ class Audiences(object):
 
         if audience_queries is None:
 
-            audience_queries, raw_data = self.get_queries_from_db()
+            audience_queries, raw_data = self.get_queries_from_db(au_domain=self.au_domain)
 
             if audience_queries is None:
                 InvalidAudienceConfiguration('No valid audiences configuration found.')
@@ -252,12 +309,13 @@ class Audiences(object):
         else:
             return None
 
-    def get_queries_from_db(self):
+    @classmethod
+    def get_queries_from_db(cls, au_domain):
 
-        data = [a.short_data for a in self.django_audience_model.objects.filter(active=True, domain=self.au_domain)]
+        data = [a.short_data for a in cls.django_audience_model.objects.filter(active=True, domain=au_domain)]
 
-        if not self.domain == '_global':
-            data.extend([a.short_data for a in self.django_audience_model.objects.filter(active=True, domain='_global')])
+        if not au_domain == '_global':
+            data.extend([a.short_data for a in cls.django_audience_model.objects.filter(active=True, domain='_global')])
 
         audience_queries = [AudienceQuery.from_raw_data(raw_data=d) for d in data]
 
@@ -266,9 +324,37 @@ class Audiences(object):
     def queries_to_cache(self,
                          audience_data):
 
-        self.redis_client.set(self.cache_audiences_key, json.dumps(audience_data), ex=self.audience_ttl)
+        self._queries_to_cache(au_domain=self.au_domain, audience_data=audience_data)
+
+    @classmethod
+    def _queries_to_cache(cls,
+                          au_domain,
+                          audience_data):
+
+        cls.redis_client.set(cls._cache_audiences_key(au_domain=au_domain), json.dumps(audience_data),
+                             ex=cls.audience_ttl)
+
+    def build_cache(self):
+
+        self._build_cache(au_domain=self.au_domain)
+
+    @classmethod
+    def _build_cache(cls, au_domain=None):
+
+        if au_domain is None or au_domain == '_global':
+            au_domains = list(set(['_global', *cls.audience_domains]))
+        else:
+            au_domains = [au_domain]
+        for d in au_domains:
+            audience_queries, raw_data = cls.get_queries_from_db(au_domain=d)
+            cls._queries_to_cache(au_domain=au_domain, audience_data=raw_data)
 
     @property
     def cache_audiences_key(self):
 
-        return f'au:{self.domain}:defs:{self.au_domain}'
\ No newline at end of file
+        return self._cache_audiences_key(au_domain=self.au_domain)
+
+    @classmethod
+    def _cache_audiences_key(cls, au_domain):
+
+        return f'au:{cls.domain}:defs:{au_domain}'
\ No newline at end of file
