@@ -220,7 +220,10 @@ def getAudiencesClass(
         redis_client,
         django_audience_model,
         django_connection_tuple,
-        audience_ttl=None):
+        django_max_func,
+        audience_ttl,
+        audience_domains,
+        default_audience_id_prefix):
 
     cls = Audiences
     cls.domain = domain
@@ -228,5 +231,8 @@ def getAudiencesClass(
     cls.django_audience_model = django_audience_model
     cls.django_connection_tuple = django_connection_tuple
     cls.audience_ttl = audience_ttl
+    cls.audience_domains = audience_domains
+    cls.default_audience_id_prefix = default_audience_id_prefix
+    cls.django_max_func = django_max_func
 
     return cls
\ No newline at end of file
